==> redpesk-cli_3.15.0.orig.tar.gz: redpesk-cli-3.15.0/cmd/boards.go <==
/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/
package cmd

import (
	"fmt"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/printer"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
)

func initBoardsCmd(rootCmd *cobra.Command) {

	// boardsCmd represents the boards command
	var boardsCmd = &cobra.Command{
		Use:   boardsCommand,
		Short: boardsCmdShortDesc,
	}

	// listCmd represents the list command
	var listCmd = &cobra.Command{
		Use:   boardsListCmd,
		Short: boardsListCmdShortDesc,
		Long:  boardsListCmdLongDesc,
		RunE:  listBoards,
	}

	// getCmd represents the get command
	var getCmd = &cobra.Command{
		Use:               boardsGetCmd,
		Short:             boardsGetCmdShortUsage,
		RunE:              getBoard,
		ValidArgsFunction: getFullListBoardsAsTab,
		Annotations:       map[string]string{"__arg_need_bck_connect__": "TRUE"},
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	listCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	getCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	// ----------------------------------------------------------

	// Add some flags as required -------------------------------
	// ----------------------------------------------------------

	// Add the group commands "boards" as subcommand of rootCmd
	rootCmd.AddCommand(boardsCmd)
	// Add the command "list" as a subcommand of boardModelsCmd
	boardsCmd.AddCommand(listCmd)
	// Add the command "get" as a subcommand of boardModelsCmd
	boardsCmd.AddCommand(getCmd)
}

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------
// getFullListBoardsAsTab gets all the boards available in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getFullListBoardsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	boardsList := []string{}

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err := bckAPI.RPBoardsGet(&boards, false); err != nil {
		return boardsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the boards list, fill the boardsList tab
	for _, board := range boards {
		boardsList = append(boardsList, board.Name)
	}

	return boardsList, cobra.ShellCompDirectiveNoFileComp
}

// getOTAListBoardsAsTab gets all the OTA boards available in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getOTAListBoardsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	boardsList := []string{}

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err := bckAPI.RPBoardsGet(&boards, false); err != nil {
		return boardsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the boards list, fill the boardsList tab
	for _, board := range boards {
		if board.BoardType == rpapiv1.RPBoardTypeOTA {
			boardsList = append(boardsList, board.Name)
		}
	}

	return boardsList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// listBoards gets and prints the list of all the available boards
// on the Redpesk backend
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func listBoards(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Prepare the function needed by the printer to get Released Projects
	getBoardsCB := func(pageNb, perPage int) (interface{}, printer.RPPrinterPaging, error) {
		// Get boards list
		boards := rpapiv1.RPBoards{}
		if err := bckAPI.RPBoardsListByPage(&boards, pageNb, perPage); err != nil {
			return boards.Boards, printer.RPPrinterPaging{}, err
		}
		// Get the paging interface from the structure retrieved
		pagingStruct, _ := printer.GetPagingFromInterface(&boards)

		return boards.Boards, pagingStruct, nil
	}

	// Prepare the structure to manage paging
	pagingStruct := printer.RPPagingToolsStruct{
		Verbose:       verbose,
		VerboseLineNb: boardDisplayListLineNb,
		GetObjCB:      getBoardsCB,
		PrintFancyCB:  displayBoards,
	}

	return RPPrinter.InteractivelyPrintPages(&pagingStruct)
}

// getBoard gets and prints the given board
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func getBoard(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the board ID wanted by the user
	brdID, err := getFirstArg(args, fmt.Sprintf(objectIDNotSetMsg, "board"), fmt.Sprintf(objectIDRequiredMsg, "board"))
	if err != nil {
		return err
	}

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err = bckAPI.RPBoardsGet(&boards, false); err != nil {
		return err
	}

	IDIsUUID := false
	brdUUID, err := uuid.FromString(brdID)
	if err == nil {
		IDIsUUID = true
	}

	// Get the board that is interesting us
	boardFound := false
	for _, board := range boards {
		if (board.Name == brdID) ||
			(IDIsUUID && utils.UUIDEqual(brdUUID, board.ID)) {
			// Print the result to the user
			err = RPPrinter.CustomPrintObject(board, verbose, displayOneBoard)
			boardFound = true
			break
		}
	}

	if !boardFound {
		err = fmt.Errorf(boardIdentifierNotFound, brdID)
	}

	return err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getBoardUUID gets the UUID of a board from either its UUID or
// its slug
//
// :param boardID: Identifier of the board (either UUID or slug)
// :type boardID: string
// :returns: uuid.UUID, error -- the UUID of the board and an error if something went bad
func getBoardUUID(boardID string) (uuid.UUID, error) {
	var err error
	boardUUID := uuid.Nil

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err = bckAPI.RPBoardsGet(&boards, false); err != nil {
		return boardUUID, err
	}

	// Check if the ID given is a UUID
	IDIsUUID := false
	brdUUID, err := uuid.FromString(boardID)
	if err == nil {
		IDIsUUID = true
	}

	// Find the board that is interesting us
	boardFound := false
	for _, board := range boards {
		if (board.Name == boardID) ||
			(IDIsUUID && utils.UUIDEqual(brdUUID, board.ID)) {
			// Print the result to the user
			boardUUID = board.ID
			boardFound = true
			break
		}
	}

	if !boardFound {
		err = fmt.Errorf(boardIdentifierNotFound, boardID)
	} else {
		err = nil
	}

	return boardUUID, err
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayBoards is a private function allowing to display
// a slice of boards in the console
//
// :param boards: tab of boards to display
// :type boards: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayBoards(boards interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Convert to the right type
	brds := boards.([]rpapiv1.RPBoard)

	// Display result
	for index, brd := range brds {
		if verbose && (index > 0) {
			fmt.Fprintln(writer)
		} else if !verbose && index == 0 {
			manageHeaderPrint(boardsListAnsHeader, writer)
		}
		// Print this project
		_printOneBoard(brd, verbose, false, writer)
	}
	return nil
}

// displayOneBoard is a private function allowing to display
// one board in the console
//
// :param board: Board to display
// :type board: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :returns: error -- an error if something went bad
func displayOneBoard(board interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Print the project
	_printOneBoard(board, verbose, true, writer)
	return nil
}

// _printOneBoard display one board available in the
// current Redpesk instance
//
// :param board: Board to print
// :type board: interface{}
// :param verbose: Tell if the print needs to be verbose or not
// :type verbose: bool
// :param isAlone: Tell if the object is alone or not
// :type isAlone: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func _printOneBoard(board interface{}, verbose, isAlone bool, writer *tabwriter.Writer) error {

	// Convert to the right type
	boardToPrint := board.(rpapiv1.RPBoard)

	if verbose {
		// Work creator if we have nothing to print
		creatorToPrint := boardToPrint.UserCreatorPseudo
		if creatorToPrint == "" {
			creatorToPrint = mainNotSetMsg
		}

		// Work IP if we have nothing to print
		IPToPrint := boardToPrint.IP
		if IPToPrint == "" {
			IPToPrint = mainNotSetMsg
		}

		// Work distribution if we have nothing to print
		distroToPrint := boardToPrint.DistributionSlug
		if distroToPrint == "" {
			distroToPrint = mainNotSetMsg
		}

		fmt.Fprintln(writer, boardsListAnsLine0, boardToPrint.ID.String())
		fmt.Fprintln(writer, boardsListAnsLine1, boardToPrint.Name)
		fmt.Fprintln(writer, boardsListAnsLine2, boardToPrint.CreatedAt.String())
		fmt.Fprintln(writer, boardsListAnsLine3, boardToPrint.UpdatedAt.String())
		fmt.Fprintln(writer, boardsListAnsLine4, creatorToPrint)
		fmt.Fprintln(writer, boardsListAnsLine5, string(boardToPrint.BoardType))
		fmt.Fprintln(writer, boardsListAnsLine6, boardToPrint.ModelName)
		fmt.Fprintln(writer, boardsListAnsLine7, IPToPrint)
		fmt.Fprintln(writer, boardsListAnsLine8, string(boardToPrint.Off))
		fmt.Fprintln(writer, boardsListAnsLine9, boardToPrint.Architecture)
		fmt.Fprintln(writer, boardsListAnsLine10, distroToPrint)

	} else {
		if isAlone {
			fmt.Fprintln(writer, boardsListAnsHeader)
		}
		fmt.Fprintln(writer, boardToPrint.ID.String(), "\t", boardToPrint.Name, "\t", string(boardToPrint.BoardType), "\t", boardToPrint.ModelName, "\t", boardToPrint.Architecture, "\t", string(boardToPrint.Off))
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

==> redpesk-cli_3.18.3.orig.tar.gz: redpesk-cli-3.18.3/cmd/boards.go <==
/*
Copyright © 2020-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage

	Licensees holding valid commercial IoT.bzh licenses may use this file in
	accordance with the commercial license agreement provided with the
	Software or, alternatively, in accordance with the terms contained in
	a written agreement between you and The IoT.bzh Company. For licensing terms
	and conditions see https://www.iot.bzh/terms-conditions. For further
	information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage

	Alternatively, this file may be used under the terms of the GNU General
	Public license version 3. This license is as published by the Free Software
	Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
	of this file. Please review the following information to ensure the GNU
	General Public License requirements will be met
	https://www.gnu.org/licenses/gpl-3.0.html.

$RP_END_LICENSE$
*/
package cmd

import (
	"fmt"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/printer"
	"git.ovh.iot/redpesk/redpesk-infra/redpesk-cli/utils"
	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

func initBoardsCmd(rootCmd *cobra.Command) {

	// boardsCmd represents the boards command
	var boardsCmd = &cobra.Command{
		Use:   boardsCommand,
		Short: boardsCmdShortDesc,
	}

	// listCmd represents the list command
	var listCmd = &cobra.Command{
		Use:   boardsListCmd,
		Short: boardsListCmdShortDesc,
		Long:  boardsListCmdLongDesc,
		RunE:  listBoards,
	}

	// getCmd represents the get command
	var getCmd = &cobra.Command{
		Use:               boardsGetCmd,
		Short:             boardsGetCmdShortUsage,
		RunE:              getBoard,
		ValidArgsFunction: getFullListBoardsAsTab,
		Annotations:       map[string]string{"__arg_need_bck_connect__": "TRUE"},
	}

	// Here you will define your flags and configuration settings.

	// Persistent Flags -----------------------------------------
	// ----------------------------------------------------------

	// Local Flags ----------------------------------------------
	listCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	getCmd.Flags().BoolP(mainVerboseFlag, mainVerboseFlagAlias, mainVerboseFlagDefault, mainVerboseFlagUsage)
	// ----------------------------------------------------------

	// Add some flags as required -------------------------------
	// ----------------------------------------------------------

	// Add the group commands "boards" as subcommand of rootCmd
	rootCmd.AddCommand(boardsCmd)
	// Add the command "list" as a subcommand of boardModelsCmd
	boardsCmd.AddCommand(listCmd)
	// Add the command "get" as a subcommand of boardModelsCmd
	boardsCmd.AddCommand(getCmd)
}

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------
// getFullListBoardsAsTab gets all the boards available in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getFullListBoardsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	boardsList := []string{}

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err := bckAPI.RPBoardsGet(&boards, ""); err != nil {
		return boardsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the boards list, fill the boardsList tab
	for _, board := range boards {
		boardsList = append(boardsList, board.Name)
	}

	return boardsList, cobra.ShellCompDirectiveNoFileComp
}

// getOTAListBoardsAsTab gets all the OTA boards available in the Redpesk
// instance and return it as an array of strings
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getOTAListBoardsAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	boardsList := []string{}

	// Get the list of every available boards
	boards := []rpapiv1.RPBoard{}
	if err := bckAPI.RPBoardsGet(&boards, ""); err != nil {
		return boardsList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the boards list, fill the boardsList tab
	for _, board := range boards {
		if board.BoardType == rpapiv1.RPBoardTypeOTA {
			boardsList = append(boardsList, board.Name)
		}
	}

	return boardsList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// listBoards gets and prints the list of all the available boards
// on the Redpesk backend
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func listBoards(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Prepare the function needed by the printer to get Released Projects
	getBoardsCB := func(pageNb, perPage int) (interface{}, printer.RPPrinterPaging, error) {
		// Get boards list
		boards := rpapiv1.RPBoards{}
		if err := bckAPI.RPBoardsListByPage(&boards, pageNb, perPage); err != nil {
			return boards.Boards, printer.RPPrinterPaging{}, err
		}
		// Get the paging interface from the structure retrieved
		pagingStruct, _ := printer.GetPagingFromInterface(&boards)

		return boards.Boards, pagingStruct, nil
	}

	// Prepare the structure to manage paging
	pagingStruct := printer.RPPagingToolsStruct{
		Verbose:       verbose,
		VerboseLineNb: boardDisplayListLineNb,
		GetObjCB:      getBoardsCB,
		PrintFancyCB:  displayBoards,
	}

	return RPPrinter.InteractivelyPrintPages(&pagingStruct, viper.GetBool(NoHeaderFlag))
}

// getBoard gets and prints the given board
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went bad
func getBoard(cmd *cobra.Command, args []string) error {

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Get the board ID wanted by the user
	brdID, err := getFirstArg(args, fmt.Sprintf(objectIDNotSetMsg, "board"), fmt.Sprintf(objectIDRequiredMsg, "board"))
	if err != nil {
		return err
	}

	// Get the board from the factory
	board, err := getBoardStruct(brdID)
	if err != nil {
		return err
	}

	return RPPrinter.CustomPrintObject(board, verbose, displayOneBoard)
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getBoardStruct allows to retrieve the full board structure
// from its name or UUID
//
// :param boardID: Name or UUID of the board that interests us
// :type boardID: string
// :returns: rpapiv1.RPBoard, error -- the board structure and an error if a problem appends
func getBoardStruct(boardID string) (rpapiv1.RPBoard, error) {
	// Check if the ID given is a valid UUID
	isValidUUID := utils.IsValidUUID(boardID)

	filterValue := ""
	if !isValidUUID {
		filterValue = "name=" + boardID
	}

	// Get the list of available boards with the right filter
	boards := []rpapiv1.RPBoard{}
	if err := bckAPI.RPBoardsGet(&boards, filterValue); err != nil {
		return rpapiv1.RPBoard{}, err
	}

	wantedIndex := -1
	// If the ID was directly an UUID, look for the right inside the list
	if isValidUUID {
		boardUUID, _ := uuid.FromString(boardID)
		for index, brd := range boards {
			if utils.UUIDEqual(brd.ID, boardUUID) {
				wantedIndex = index
				break
			}
		}
	} else {
		wantedIndex = 0
	}

	if len(boards) == 0 || wantedIndex == -1 {
		return rpapiv1.RPBoard{}, fmt.Errorf(boardIdentifierNotFound, boardID)
	}

	// We got correctly the board
	return boards[wantedIndex], nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayBoards is a private function allowing to display
// a slice of boards in the console
//
// :param boards: tab of boards to display
// :type boards: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayBoards(boards interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Convert to the right type
	brds := boards.([]rpapiv1.RPBoard)

	// Display result
	for index, brd := range brds {
		if verbose && (index > 0) {
			fmt.Fprintln(writer)
		} else if !verbose && index == 0 {
			manageHeaderPrint(boardsListAnsHeader, writer)
		}
		// Print this project
		_printOneBoard(brd, verbose, false, writer)
	}
	return nil
}

// displayOneBoard is a private function allowing to display
// one board in the console
//
// :param board: Board to display
// :type board: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :returns: error -- an error if something went bad
func displayOneBoard(board interface{}, verbose bool, writer *tabwriter.Writer) error {
	// Print the project
	_printOneBoard(board, verbose, true, writer)
	return nil
}

// _printOneBoard display one board available in the
// current Redpesk instance
//
// :param board: Board to print
// :type board: interface{}
// :param verbose: Tell if the print needs to be verbose or not
// :type verbose: bool
// :param isAlone: Tell if the object is alone or not
// :type isAlone: bool
// :param writer: Writer to use to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func _printOneBoard(board interface{}, verbose, isAlone bool, writer *tabwriter.Writer) error {

	// Convert to the right type
	boardToPrint := board.(rpapiv1.RPBoard)

	if verbose {
		// Work creator if we have nothing to print
		creatorToPrint := boardToPrint.UserCreatorPseudo
		if creatorToPrint == "" {
			creatorToPrint = mainNotSetMsg
		}

		// Work IP if we have nothing to print
		IPToPrint := boardToPrint.IP
		if IPToPrint == "" {
			IPToPrint = mainNotSetMsg
		}

		// Work distribution if we have nothing to print
		distroToPrint := boardToPrint.DistributionSlug
		if distroToPrint == "" {
			distroToPrint = mainNotSetMsg
		}

		fmt.Fprintln(writer, boardsListAnsLine0, boardToPrint.ID.String())
		fmt.Fprintln(writer, boardsListAnsLine1, boardToPrint.Name)
		fmt.Fprintln(writer, boardsListAnsLine2, boardToPrint.CreatedAt.String())
		fmt.Fprintln(writer, boardsListAnsLine3, boardToPrint.UpdatedAt.String())
		fmt.Fprintln(writer, boardsListAnsLine4, creatorToPrint)
		fmt.Fprintln(writer, boardsListAnsLine5, string(boardToPrint.BoardType))
		fmt.Fprintln(writer, boardsListAnsLine6, boardToPrint.ModelName)
		fmt.Fprintln(writer, boardsListAnsLine7, IPToPrint)
		fmt.Fprintln(writer, boardsListAnsLine8, string(boardToPrint.Off))
		fmt.Fprintln(writer, boardsListAnsLine9, boardToPrint.Architecture)
		fmt.Fprintln(writer, boardsListAnsLine10, distroToPrint)

	} else {
		if isAlone {
			fmt.Fprintln(writer, boardsListAnsHeader)
		}
		fmt.Fprintln(writer, boardToPrint.ID.String(), "\t", boardToPrint.Name, "\t", string(boardToPrint.BoardType), "\t", boardToPrint.ModelName, "\t", boardToPrint.Architecture, "\t", string(boardToPrint.Off))
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------
